# Created by george zhao, who is working for NOKIA, 2010-2012, 2014-2018.												#
# This lib is based on python 3.6.5																						#

MSG_FLAGS_UNSET = 0x0
MSG_COMMAND_CODE_UNSET = 0
MSG_APP_ID_UNSET = 0
MSG_HBH_ID_UNSET = 0x0
MSG_E2E_ID_UNSET = 0x0

MSG_VERSION_BUFF_LEN = 1
MSG_LENGTH_BUFF_LEN = 3
MSG_FLAGS_BUFF_LEN = 1
MSG_COMMAND_CODE_BUFF_LEN = 3
MSG_APPLICATION_ID_BUFF_LEN = 4
MSG_HBH_BUFF_LEN = 4
MSG_E2E_BUFF_LEN = 4
MSG_HEADER_BUFF_LEN = MSG_VERSION_BUFF_LEN + MSG_LENGTH_BUFF_LEN + MSG_FLAGS_BUFF_LEN + MSG_COMMAND_CODE_BUFF_LEN + MSG_APPLICATION_ID_BUFF_LEN + MSG_HBH_BUFF_LEN + MSG_E2E_BUFF_LEN
