# Created by george zhao, who is working for NOKIA, 2010-2012, 2014-2018.												#
# This lib is based on python 3.6.5																						#

# Decode Bytes Order																								#
BYTES_ORDER_BIGENDIAN = 'big'
BYTES_ORDER_LITTLEENDIAN = 'little'
DEFAULT_BYTES_ORDER = BYTES_ORDER_BIGENDIAN
BYTES_ORDER_2_BIGENDIAN = '>'
BYTES_ORDER_2_LITTLEDIAN = '<'
DEFAULT_BYTES_ORDER_2 =  BYTES_ORDER_2_BIGENDIAN

# Const for Bytes class.																							#
NULL_BYTE = b'\x00'
ZERO_LEN_BYTES = b'' 

